// SineGeneratorExampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SineGeneratorExample.h"
#include "SineGeneratorExampleDlg.h"
#include ".\sinegeneratorexampledlg.h"
#include <rssiup.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CSineGeneratorExampleDlg dialog
#define CHECKERR(fce) \
	if (status = (fce), status < 0) \
	{ \
		CString	message("");\
		rssiup_error_message (io, status, error_message);\
        message.Format("Function Call Status: 0x%08X, %s\n", status, error_message);\
        if (io != 0) \
		{ \
			do {\
				rssiup_error_query(io, &status, error_message);\
				if (status < 0)\
					message.AppendFormat("Instrument Error: 0x%X, %s\n", status, error_message);\
			} while (status != 0); \
		}\
		MessageBox(message, "Error"); \
		goto Error; \
	} \
	else


CSineGeneratorExampleDlg::CSineGeneratorExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSineGeneratorExampleDlg::IDD, pParent)
	, m_ResourceName(_T("USB::0xAAD::0x8::000000"))
	, m_IDQuery(TRUE)
	, m_Reset(TRUE)
	, m_ChannelIndex(0)
	, m_Frequency1(1000.0)
	, m_Frequency2(2000.0)
	, m_Phase1(0.0)
	, m_Phase2(0.0)
	, m_Amplitude1(1.0)
	, m_Amplitude2(2.0)
	, m_DriverRev(_T(""))
	, m_FirmwareRev(_T(""))
	, m_OutputEnable(TRUE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSineGeneratorExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_RESOURCE, m_ResourceName);
	DDV_MaxChars(pDX, m_ResourceName, 40);
	DDX_Check(pDX, IDC_IDQUERY, m_IDQuery);
	DDX_Check(pDX, IDC_RESET, m_Reset);
	DDX_CBIndex(pDX, IDC_CHANNEL_LIST, m_ChannelIndex);
	DDX_Text(pDX, IDC_FREQUENCY2, m_Frequency2);
	DDX_Text(pDX, IDC_FREQUENCY1, m_Frequency1);
	DDX_Text(pDX, IDC_PHASE1, m_Phase1);
	DDV_MinMaxDouble(pDX, m_Phase1, -180, 180);
	DDX_Text(pDX, IDC_PHASE2, m_Phase2);
	DDV_MinMaxDouble(pDX, m_Phase2, -180, 180);
	DDX_Text(pDX, IDC_AMPLITUDE1, m_Amplitude1);
	DDX_Text(pDX, IDC_AMPLITUDE2, m_Amplitude2);
	DDX_Text(pDX, IDC_DRIVER_REV, m_DriverRev);
	DDX_Text(pDX, IDC_FIRMWARE_REV, m_FirmwareRev);
	DDX_Check(pDX, IDC_ENABLED, m_OutputEnable);
}

BEGIN_MESSAGE_MAP(CSineGeneratorExampleDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_RUN, OnBnClickedRun)
END_MESSAGE_MAP()


// CSineGeneratorExampleDlg message handlers

BOOL CSineGeneratorExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSineGeneratorExampleDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSineGeneratorExampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CSineGeneratorExampleDlg::OnBnClickedRun()
{
	ViSession	io = 0;
	ViInt32		status = 0;
	ViInt32		channel = -1;
	ViChar		driver_rev[256];
	ViChar		firmware_rev[256];
	ViChar		error_message[256];

	UpdateData();

	/* Initialize driver */
	CHECKERR( rssiup_init((ViRsrc) m_ResourceName.GetBuffer(), m_IDQuery, m_Reset, &io));
	m_ResourceName.ReleaseBuffer();

	/* Get Driver and Firmware Revision */
	CHECKERR( rssiup_revision_query (io, driver_rev, firmware_rev));

	m_DriverRev = CString(driver_rev);
	m_FirmwareRev = CString(firmware_rev);
	UpdateData(FALSE);
	
	/* Select Output Device and Sampling Frequency of the generator */
	CHECKERR( rssiup_SetGeneratorDeviceSelector (io, RSSIUP_GEN_A49));

	switch (m_ChannelIndex)
	{
	case 1:
		channel = RSSIUP_CHA;
		break;
	case 2:
		channel = RSSIUP_CHB;
		break;
	case 0:
	default:
		channel = RSSIUP_CHAB;
		break;
	}
	/* Set Generator ground */
	CHECKERR( rssiup_SetAnalogGeneratorFloatGround (io, channel, VI_FALSE));
	
	/* Configure Analog Range Mode and Value */
	CHECKERR( rssiup_SetAnalogGeneratorRangeValue (io, channel, RSSIUP_RANGE_MODE_AUTO, 0.0, RSSIUP_UNIT_V));
	
	/* Configure Frequency and Phase paraneters for sine waveform */
	CHECKERR( rssiup_SetGeneratorFunctionSineShape (io,
													m_Frequency1, RSSIUP_UNIT_HZ,
													m_Phase1, RSSIUP_UNIT_DEG,
													m_Frequency2, RSSIUP_UNIT_HZ,
													m_Phase2, RSSIUP_UNIT_DEG));
	
	switch (channel)
	{
	case RSSIUP_CHAB:
		/* Configure Amplitude and Offset for sine waveform on channel A */
		CHECKERR( rssiup_SetGeneratorFunctionSineDimension (io, RSSIUP_CHA,
															0.0, RSSIUP_UNIT_V,
															m_Amplitude1, RSSIUP_UNIT_V,
															0.0, RSSIUP_UNIT_V));
		
		/* Configure Amplitude and Offset for sine waveform on channel B */
		CHECKERR( rssiup_SetGeneratorFunctionSineDimension (io, RSSIUP_CHB,
															0.0, RSSIUP_UNIT_V,
															0.0, RSSIUP_UNIT_V,
															m_Amplitude2, RSSIUP_UNIT_V));
		break;
	case RSSIUP_CHA:
		/* Configure Amplitude and Offset for sine waveform on channel A */
		CHECKERR( rssiup_SetGeneratorFunctionSineDimension (io, RSSIUP_CHA,
															0.0, RSSIUP_UNIT_V,
															m_Amplitude1, RSSIUP_UNIT_V,
															0.0, RSSIUP_UNIT_V));
		break;
	case RSSIUP_CHB:
		/* Configure Amplitude and Offset for sine waveform on channel B */
		CHECKERR( rssiup_SetGeneratorFunctionSineDimension (io, RSSIUP_CHB,
															0.0, RSSIUP_UNIT_V,
															0.0, RSSIUP_UNIT_V,
															m_Amplitude2, RSSIUP_UNIT_V));
		break;
	}
	
	/* Select generator function to switch on */
	CHECKERR( rssiup_SetGeneratorFunctionState (io, channel, RSSIUP_GEN_FUNC_SINE,
												m_OutputEnable));
	
	/* Turn on Generator Output */
	CHECKERR( rssiup_SetAnalogGeneratorOutput (io, channel, VI_TRUE));

Error:
	rssiup_close(io);
}
